# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"clock": "01weylheisenberg.ipynb",
         "shift": "01weylheisenberg.ipynb",
         "displace": "01weylheisenberg.ipynb",
         "weyl_heisenberg_indices": "01weylheisenberg.ipynb",
         "displacement_operators": "01weylheisenberg.ipynb",
         "weyl_heisenberg_states": "01weylheisenberg.ipynb",
         "weyl_heisenberg_povm": "01weylheisenberg.ipynb",
         "to_weyl_heisenberg_basis": "01weylheisenberg.ipynb",
         "from_weyl_heisenberg_basis": "01weylheisenberg.ipynb",
         "load_fiducial": "02sics.ipynb",
         "sic_states": "02sics.ipynb",
         "sic_povm": "02sics.ipynb",
         "sic_gram": "02sics.ipynb",
         "hoggar_fiducial": "02sics.ipynb",
         "hoggar_indices": "02sics.ipynb",
         "hoggar_povm": "02sics.ipynb",
         "povm_gram": "03povm.ipynb",
         "povm_phi": "03povm.ipynb",
         "dm_probs": "03povm.ipynb",
         "probs_dm": "03povm.ipynb",
         "conditional_probs": "03povm.ipynb",
         "quantum_inner_product": "03povm.ipynb",
         "tensor_povm": "03povm.ipynb",
         "upgrade_povm": "03povm.ipynb",
         "apply_dims": "03povm.ipynb",
         "implement_povm": "03povm.ipynb",
         "discriminator_povm": "03povm.ipynb",
         "apply_kraus": "04kraus.ipynb",
         "partial_trace_kraus": "04kraus.ipynb",
         "povm_map": "04kraus.ipynb"}

modules = ["weyl_heisenberg.py",
           "sics.py",
           "povm.py",
           "kraus.py"]

doc_url = "https://heyredhat.github.io/qbism/qbism/"

git_url = "https://github.com/heyredhat/qbism/tree/master/"

def custom_doc_links(name): return None
